function z = solveLassoSmooth( y, X, W, beta_0, lambda, mu )
%SOLVEDFUSEDLASSO Solves the temporal smoothing weighted l1 problem
%
%           0.5 * (y - X*beta)'*(y - X*beta) + lambda * ||beta||_1 + mu *
%           ||beta-beta_0||_1
%
%   where ||beta||_1 is the L_1 norm i.e., ||beta||_1 = sum(abs( beta ))
%
%   We use the method proposed by Fu et. al based on single co-ordinate
%   descent. For more details see GP's notes or the following paper:
%
%   Penalized Regressions: The Bridge Versus the Lasso
%   Wenjiang J. FU, Journal of Computational and Graphical Statistics,
%   Volume 7, Number 3, Pages 397?416, 1998
%
%   This is an update of the code by Gautam Pendse
%
% Julien LESOUPLE 12 sep 2018
%
%==========================================================================
%
%==========================================================================
%
%               USAGE:
%
%               z = solveFusedLasso( y, X, beta_0, lambda, mu )
%
%==========================================================================
%
%==========================================================================
%
%               INPUTS:
%
%       =>      y = n by 1 response vector
%
%       =>      X = n by p design matrix
%
%       => lambda = regularization parameter for L1 penalty
%
%==========================================================================
%
%==========================================================================
%
%               OUTPUTS:
%
%       =>      z.X = supplied design matrix
%
%       =>      z.y = supplied response vector
%
%       => z.lambda = supplied regularization parameter for L1 penalty
%
%       =>   z.beta = computed L1 regularized solution
%
%==========================================================================
%
%
%==========================================================================

%==========================================================================
%               check input args
% tic
% if ( nargin ~= 3 )
%     disp('Usage: z = solveLasso( y, X, lambda )');
%     z = [];
%     return;
% end

% check size of y
[n1, p1] = size(y);

% is y a column vector?
if ( p1 ~= 1 )
    disp('y must be a n by 1 vector!!');
    z = [];
    return;
end

% check size of X
[n2,p2] = size(X);

% does X have the same number of rows as y?
if ( n2 ~= n1 )
    disp('X must have the same number of rows as y!!!');
    z = [];
    return;
end

% check size of beta
[n3,p3] = size(beta_0);

% is beta_0 a column vector?
if ( p3 ~= 1 )
    disp('beta_0 must be a n by 1 vector!!');
    z = [];
    return;
end

% does beta_0 have the same number of rows as columns of X?
if ( p2 ~= n3 )
    disp('beta_0 must have the same number of rows as the number of columns of X!!!');
    z = [];
    return;
end

% make sure lambda > 0
if ( lambda < 0 )
    disp('lambda must be >= 0!');
    z = [];
    return;
end

% make sure mu > 0
if ( mu < 0 )
    disp('mu must be >= 0!');
    z = [];
    return;
end

% get size of X
[n, p] = size(X);

% mu is considered as a vector. this can be useful when the number of
% satellites changes over time (mu=0 is equivalent to the avoid smoothing,
% for the satellites that where not there at the time instant before, and
% thus who don't have an estimated bias at the previous time instant 

if length(mu)== 1
    mu=mu*ones(n,1);
end

%==========================================================================

%==========================================================================
%               initialize the Lasso solution

% This assumes that the penalty is lambda * beta'*beta instead of lambda * ||beta||_1
%    beta = (X'*X + 2*lambda) \ (X'*y);
beta=zeros(p,1);
%==========================================================================


%==========================================================================
%               start while loop

% convergence flag
found = 0;

% convergence tolerance
TOL = 1e-4;
k=0;
while( found == 0 )
    k=k+1;
    % save current beta
    beta_old = beta;
    
    % optimize elements of beta one by one
    for i = 1:p
        
        % optimize element i of beta
        
        % get ith col of X
        xi = X(:,i);
        
        % get residual excluding ith col
        yi = (y - X*beta) + xi*beta(i);
        
        % calulate xi'*yi and see where it falls
        deltai = (xi'*yi); % 1 by 1 scalar
        if mu(i)*W(i)==0
            %             if ( deltai > lambda*W(i)  )
            %
            %                 beta(i) = ( deltai - lambda*W(i)   )/(xi'*xi);
            %
            %             elseif ( deltai < -lambda*W(i)   )
            %
            %
            %                 beta(i) = ( deltai + lambda*W(i)  )/(xi'*xi);
            %
            %             else
            %
            %                 beta(i) = 0;
            %
            %             end
            
            if ( deltai < -lambda*W(i) )
                
                beta(i) = ( deltai + lambda*W(i) )/(xi'*xi);
                
            elseif ( deltai > lambda*W(i) )
                
                beta(i) = ( deltai - lambda*W(i) )/(xi'*xi);
                
            else
                
                beta(i) = 0;
                
            end
        else
            %1
            if ( deltai > lambda*W(i) + mu(i)*W(i) ) && (deltai > (xi'*xi)*beta_0(i)+lambda*W(i)+mu(i)*W(i))
                
                beta(i) = ( deltai - lambda*W(i) - mu(i)*W(i) )/(xi'*xi);
                
                %2
            elseif ( deltai > lambda*W(i) - mu(i)*W(i) ) && (deltai < (xi'*xi)*beta_0(i)+lambda*W(i)-mu(i)*W(i))
                
                beta(i) = ( deltai - lambda*W(i) + mu(i)*W(i) )/(xi'*xi);
                
                %3
            elseif ( deltai < -lambda*W(i) + mu(i)*W(i)  ) && (deltai > (xi'*xi)*beta_0(i)-lambda*W(i)+mu(i)*W(i))
                
                beta(i) = ( deltai + lambda*W(i) -mu(i)*W(i) )/(xi'*xi);
                
                %4
            elseif ( deltai < -lambda*W(i) - mu(i)*W(i)  ) && (deltai < (xi'*xi)*beta_0(i)-lambda*W(i)-mu(i)*W(i))
                
                beta(i) = ( deltai + lambda*W(i) + mu(i)*W(i) )/(xi'*xi);
                
            elseif beta_0(i)~=0 && abs(xi'*xi*beta_0(i)-deltai+lambda*W(i)*sign(beta_0(i)))<=mu(i)*W(i)
                
                beta(i)=beta_0(i);
            elseif beta_0(i)~=0 && abs(deltai+mu(i)*W(i)*sign(beta_0(i)))<=lambda*W(i)
                
                beta(i)=0;
                
            elseif beta_0(i)==0 && abs(deltai)<=lambda*W(i)+mu(i)*W(i)
                
                beta(i)=0;
            else
                
                error('Prox doesn''t exist here');
                
            end
            
        end
        
        
    end
    
    % check difference between beta and beta_old
    if ( max(abs(beta - beta_old)) <= TOL )
        found = 1;
    end
    objective(k)=(1/2)*norm(y-X*beta,2)^2+lambda*norm(W.*beta,1)+norm(W.*mu.*(beta-beta_0),1);
end
%==========================================================================

%==========================================================================
%   save outputs

z.X = X;
z.y = y;
z.lambda = lambda;
%     z.time=toc;
z.beta = beta;
z.nb_iter=k;
z.objective=objective;
%==========================================================================


end
